(ns tservice.plugins.xps2pdf
  (:require [clojure.data.json :as json]
            [clojure.java.io :as io]
            [clojure.spec.alpha :as s]
            [clojure.string :as str]
            [clojure.tools.logging :as log]
            [me.raynes.fs :as fs]
            [spec-tools.core :as st]
            [tservice.api.config :refer [get-workdir get-tservice-workdir]]
            [tservice.api.task :refer [make-plugin-metadata make-events-init publish-event!]]
            [tservice.api.storage.fs :as fs-lib]
            [tservice.plugins.xps2pdf.xps :as xps-lib]))

;;; ------------------------------------------------ Event Specs ------------------------------------------------
(s/def ::zip_mode
  (st/spec
   {:spec                boolean?
    :type                :boolean
    :description         "ZIP Mode?"
    :swagger/default     true
    :reason              "The zip-mode must be boolean."}))

(s/def ::pdf_mode
  (st/spec
   {:spec                boolean?
    :type                :boolean
    :description         "PDF Mode?"
    :swagger/default     true
    :reason              "The pdf-mode must be boolean."}))

(s/def ::filepath
  (st/spec
   {:spec                (s/and string? #(re-matches #"^file:\/\/(\/|\.\/)[a-zA-Z0-9_]+.*" %))
    :type                :string
    :description         "File path for covertor."
    :swagger/default     nil
    :reason              "The filepath must be string."}))

(def xps2pdf-params-body
  "A spec for the body parameters."
  (s/keys :req-un [::filepath]
          :opt-un [::pdf_mode ::zip_mode]))

;;; ------------------------------------------------ Event Metadata ------------------------------------------------
(def metadata
  (make-plugin-metadata
   {:name "xps2pdf"
    :params-schema xps2pdf-params-body
    :handler (fn [{:keys [filepath]}]
               (let [tservice-workdir (get-tservice-workdir)
                     from-path (if (re-matches #"^file:\/\/\/.*" filepath)
                                 ; Absolute path with file://
                                 (str/replace filepath #"^file:\/\/" "")
                                 (fs-lib/join-paths tservice-workdir (str/replace filepath #"^file:\/\/" "")))
                     from-files (if (fs-lib/directory? from-path)
                                  (filter #(fs-lib/regular-file? %)
                                          (map #(.getPath %) (file-seq (io/file from-path))))
                                  [from-path])
                     workdir (get-workdir)
                     zip-path (fs-lib/join-paths workdir "merged.zip")
                     pdf-path (fs-lib/join-paths workdir "merged.pdf")
                     log-path (fs-lib/join-paths workdir "log")]
                 (fs-lib/create-directories! workdir)
                 ; Launch the batchxps2pdf-convert
                 (spit (fs-lib/join-paths workdir log-path) (json/write-str {:status "Running" :msg ""}))
                 (publish-event! "batchxps2pdf" {:from-files from-files :to-dir workdir})
                 {:files [pdf-path
                          zip-path]
                  :log log-path}))
    :plugin-type :ToolPlugin
    :response-type :data2files}))

;;; ------------------------------------------------ Event Processing ------------------------------------------------

(defn- xps2pdf! [from to]
  (log/info "Converted xps to pdf: " from to)
  (let [out (xps-lib/xps2pdf from to)
        log-path (fs-lib/join-paths (fs-lib/parent-path to) "log")
        exit (:exit out)
        status (if (>= exit 0) "Success" "Error")
        msg (if (>= exit 0) (:err out) (:out out))
        log (json/write-str {:status status :msg msg})]
    (spit log-path log)))

(defn- batch-xps2pdf! [{:keys [from-files to-dir]}]
  (log/info "Converted xps files in a zip to pdf files.")
  (let [to-files (vec (map #(fs-lib/join-paths to-dir (str (fs/base-name % ".xps") ".pdf")) from-files))
        zip-path (fs-lib/join-paths to-dir "merged.zip")
        pdf-path (fs-lib/join-paths to-dir "merged.pdf")]
    (doall (pmap #(xps2pdf! %1 %2) from-files to-files))
    (fs-lib/zip-files to-files zip-path)
    (fs-lib/merge-pdf-files to-files pdf-path)))

;;; --------------------------------------------------- Lifecycle ----------------------------------------------------

(def events-init
  "Automatically called during startup; start event listener for xps2pdf events."
  (make-events-init "batchxps2pdf" batch-xps2pdf!))
